@@ -151,24 +151,24 @@ strftime(char *s, size_t maxsize, const char *format, const struct tm *timeptr)
 #endif /* POSIX_SEMANTICS */
 
 	/* various tables, useful in North America */
-	static char *days_a[] = {
+	static const char *days_a[] = {
 		"Sun", "Mon", "Tue", "Wed",
 		"Thu", "Fri", "Sat",
 	};
-	static char *days_l[] = {
+	static const char *days_l[] = {
 		"Sunday", "Monday", "Tuesday", "Wednesday",
 		"Thursday", "Friday", "Saturday",
 	};
-	static char *months_a[] = {
+	static const char *months_a[] = {
 		"Jan", "Feb", "Mar", "Apr", "May", "Jun",
 		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
 	};
-	static char *months_l[] = {
+	static const char *months_l[] = {
 		"January", "February", "March", "April",
 		"May", "June", "July", "August", "September",
 		"October", "November", "December",
 	};
-	static char *ampm[] = { "AM", "PM", };
+	static const char *ampm[] = { "AM", "PM", };
 
 	if (s == NULL || format == NULL || timeptr == NULL || maxsize == 0)
 		return 0;
